package com.app.component.sysconfig.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.app.component.sysconfig.dao.SysConfigDao;
import com.app.component.sysconfig.model.SysConfig;
import com.app.component.sysconfig.service.SysConfigService;
import com.app.framework.core.support.auth.LoginUser;
import com.app.framework.core.support.service.impl.CrudBaseServiceSupport;
import com.app.framework.core.support.view.grid.Paging;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.List;

/**
 * 系统配置管理
 * 重写缓存实现:根据配置名称
 */
@Service
public class SysConfigServiceImpl extends CrudBaseServiceSupport<SysConfigDao, SysConfig> implements SysConfigService {

    /**
     * 重写缓存实现:根据配置名称
     *
     * @param id 配置名称
     * @return
     */
    public SysConfig cacheEntityFromDb(final Serializable id) {
        LambdaQueryWrapper<SysConfig> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SysConfig::getCfgName, id);
        return this.getOne(wrapper);
    }

    public List<SysConfig> getPageList(SysConfig view, Paging paging) {
        LambdaQueryWrapper<SysConfig> wrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotBlank(view.getCfgName())) {
            wrapper.like(SysConfig::getCfgName, view.getCfgName());
        }
        wrapper.orderByDesc(SysConfig::getCreateTime);
        return this.pageList(wrapper, paging);
    }

    /**
     * 保存配置项
     *
     * @param cfg 配置项
     */
    public void saveCfg(SysConfig cfg, LoginUser loginUser) {
        if (cfg.getOid() == null) {
            this.save(cfg);
        } else {
            this.updateById(cfg);
        }
        //更新缓存
//        this.deleteFromCache(cfg.getCfgName());
    }

    /**
     * 根据配置项名称获取配置
     *
     * @param cfgName 配置项名称
     */
    public SysConfig get(String cfgName){
        return this.cacheEntityFromDb(cfgName);
    }

    /**
     * 根据配置项名称获取配置
     *
     * @param cfgName 配置项名称
     */
    public JSONObject getJson(String cfgName) {
        SysConfig cfg = this.cacheEntityFromDb(cfgName);
        if (cfg != null && StringUtils.isNotBlank(cfg.getCfgValue())) {
            return JSONObject.parseObject(cfg.getCfgValue());
        }
        return null;
    }

    /**
     * 根据配置项名称获取配置
     *
     * @param cfgName 配置项名称
     * @param tClass 返回对象的Class
     */
    public <T> T getJson(String cfgName, Class<T> tClass) {
        SysConfig cfg = this.cacheEntityFromDb(cfgName);
        if (cfg != null && StringUtils.isNotBlank(cfg.getCfgValue())) {
            return JSONObject.parseObject(cfg.getCfgValue(), tClass);
        }
        return null;
    }

    /**
     * 根据配置项名称获取配置
     *
     * @param cfgName 配置项名称
     */
    public String getString(String cfgName) {
        SysConfig cfg = this.cacheEntityFromDb(cfgName);
        if (cfg != null) {
            return cfg.getCfgValue();
        }
        return null;
    }

    /**
     * 根据配置项名称获取配置
     *
     * @param cfgName 配置项名称
     */
    public Integer getInteger(String cfgName) {
        SysConfig cfg = this.cacheEntityFromDb(cfgName);
        if (cfg != null && StringUtils.isNotBlank(cfg.getCfgValue())) {
            return Integer.parseInt(cfg.getCfgValue());
        }
        return null;
    }
}